<?php
////////////////////////////////////////////////////
// Classe de gnration d'un arbre de Formulaires //
// Cration : FG 18/02/2011                       //
////////////////////////////////////////////////////

class FormsTreeXML
{
	public $root;

	private $dom;			// DOM interne
	private $xpath;			// pour requte sur le DOM interne
	private $xpathData;		// pour requte sur le DOM externe des donnes
	private $xpathTemplate;	// pour requte sur le DOM externe de l'arbre des templates

	function __construct($contenu=null)
    {
        $this->dom = new DOMDocument('1.0', 'UTF-8');
		$this->dom->formatOutput = false;
		$this->dom->preserveWhiteSpace = false;

		if($contenu==null)
		{
			$this->root = $this->dom->createElement('databind');
			$this->dom->appendChild($this->root);
		}
		else
		{
			$this->dom->loadXML($contenu);
			$this->root = $this->dom->firstChild;
		}

		$this->xpath = new DOMXpath($this->dom);
		$this->xpathData = null;
		$this->xpathTemplate = null;
	}

///////////////////////////
// Fonctions utilitaires //
///////////////////////////

	// Gnre l'arbre XML sous forme de chaine de caractres
	public function getXML()
	{
		return $this->dom->saveXML();
	}

	// Formate la sortie en XML
	public function printXML()
	{
		Header('Content-type: text/xml');
		print $this->dom->saveXML();
	}

	// Utilitaire de recherche xpath
	private function findNodeList($req, $xpath, $node=null)
	{
		if($xpath==null)
			return new DOMNodeList();  // retourne une liste vide

		if($node!=null)
			return $xpath->query($req, $node); // recherche  partir du noeud prcis
		else
			return $xpath->query($req); // sinon  partir de la racine
	}

	// Retourne le 1er noeud trouv sur le DOM interne ayant les attributs de recherche prciss
	public function findNode($atr)
	{
		if($this->xpath==null)
			$this->xpath = new DOMXPath($this->dom);

		$req = '//*';
		foreach($atr as $name=>$value)
			$req .= "[@".$name."='$value']";
			
		$nodeList = $this->findNodeList($req, $this->xpath);

		return $nodeList->item(0);
	}

	// Ajout des noeuds row/item et des attributs de item
	public function addRowItemTree($parent, $attributes=null)
    {
		$row = $this->dom->createElement('row');
		$item = $this->dom->createElement('item');

		if($attributes!=null)
		{
			foreach($attributes as $atr=>$val)
				$item->setAttribute($atr, $val);
		}

		$row->appendChild($item);

		return $parent->appendChild($row);
    }

/////////////////////////////////////
// Fonctions de gestion de l'arbre //
//  Travaille sur les templates    //
/////////////////////////////////////

	// Importation des formulaires de domTemplate en suivant l'arbre et en important les donnes depuis domData
	public function importNodeList($domTemplate, $domData)
	{
		if($this->xpathData==null && $domData!=null)
			$this->xpathData = new DOMXPath($domData);

		if($this->xpathTemplate==null && $domTemplate!=null)
			$this->xpathTemplate = new DOMXPath($domTemplate);

		$parent = $this->root->firstChild;	
		foreach($domTemplate->firstChild->childNodes as $nodeTemplate)
			$this->importNode($parent, $nodeTemplate, null);
	}

	// Import d'un noeud et constitution des ses attributs
	private function importNode($parent, $nodeTemplate, $dataNode)
	{
		$name = $nodeTemplate->attributes->getNamedItem('name')->nodeValue;
		$repeated = $nodeTemplate->attributes->getNamedItem('repeated')->nodeValue;

		$caption = $nodeTemplate->attributes->getNamedItem('caption')->nodeValue;
		if($caption[0]='$')
			$caption = $GLOBALS[substr($caption,1)];

		$can_delete = 'false';
		if($repeated=='true')
		{
			$can_create = 'true';
			$can_modify = 'false';
			$icon = 'style/imgs/icons/histo.gif';
		}
		else
		{
			$can_create = 'false';
			$can_modify = 'true';
			$icon = 'style/imgs/icons/document.png';
		}

		if($dataNode!=null)
			$idPere = $dataNode->attributes->getNamedItem('id')->nodeValue;
		else
			$idPere = '0';

		$atr = array(
			'id' => $name,
			'caption' => $caption,
			'onClick' => "enable_button($can_create, $can_modify, $can_delete);",
			'value' => "name=$name&id=$name&idPere=$idPere",
			'icon' => $GLOBALS['DEFAULT_PATH'].$icon,
			//'open' => $nodeTemplate->hasChildNodes()==true ? '1' : '0'
			'open' => '-1'
		);

		$newNode = $this->addRowItemTree($parent, $atr);

		if($repeated=='true')
		{
			$req = "form[@name='$name']";
			$dataNodeList = $this->findNodeList($req, $this->xpathData, $dataNode);

			$this->importDataNodes($newNode, $nodeTemplate, $dataNodeList);
		}

		return $newNode;
	}

	// Import des noeud fils (formulaires saisis)
	private function importDataNodes($parent, $nodeTemplate, $dataNodeList)
	{
		$name = $nodeTemplate->attributes->getNamedItem('name')->nodeValue;

		$itemCaption = $nodeTemplate->attributes->getNamedItem('itemCaption')->nodeValue;
		if($itemCaption && $itemCaption[0]='$')
			$itemCaption = $GLOBALS[substr($itemCaption,1)];

		foreach($dataNodeList as $rang=>$dataNode)
		{
			$id = $dataNode->attributes->getNamedItem('id')->nodeValue;
			$idPere = $dataNode->parentNode->attributes->getNamedItem('id')->nodeValue;
			$atr = array(
				'id' => $id,
				'caption' => $itemCaption.'_'.($rang+1),
				'onClick' => "enable_button(false, true, true);",
				'value' => "name=$name&id=$id&idPere=$idPere",
				'icon' => $GLOBALS['DEFAULT_PATH'].'style/imgs/icons/document.png',
				'open' => '0'
			);

			$newNode = $this->addRowItemTree($parent, $atr);

			foreach($nodeTemplate->childNodes as $nodeTpl)
				$this->importNode($newNode, $nodeTpl, $dataNode);
		}
	}

////////////////////////////////////////
// Fonctions de gestion de Formulaire //
//  Travaille sur le fichier rsultat //
////////////////////////////////////////

	// Ajout un noeud 'form' de type $type au pere prcis et gnre son id hirarchique
	public function createForm($type, $pere, $repeat)
	{
		$form = $this->dom->createElement('form');

		if($repeat)
		{
			if($pere==null)
			{
				$pere = $this->root;
				$id = $type;
			}
			else
				$id = $pere->attributes->getNamedItem('id')->nodeValue;

			$rang = ($this->findNodeList("form[@name='$type']", $this->xpath, $pere)->length) + 1;
			$id .= '_'. $rang;
		}
		else
		{
			if($pere==null)
			{
				$pere = $this->root;
				$id = $type;
			}
			else
				$id = $pere->attributes->getNamedItem('id')->nodeValue .'_'.$type;
		}

		$form->setAttribute('id', $id);
		$form->setAttribute('name', $type);

		return $pere->appendChild($form);
	}

	// Ajout d'un noeud form (= formulaire de donnes) par clonage
	public function addForm($node)
	{
		$newNode = $this->dom->importNode($node, true);
		return $this->root->appendChild($newNode);
	}

	// Supprime un noeud et ses fils dont l'id est pass en paramtre
	public function deleteForm($id)
	{
		$form = $this->findNode(array('id' => $id));
		if($form)
		{
			$this->deleteFields($form);
			$parent = $form->parentNode;
			$parent->removeChild($form);
		}
	}

	// Ajout des noeud constituant les champs d'un formulaire passs en paramtre
	public function addFieldList($list, $form)
	{
		foreach($list as $name=>$value)
		{
			$field = $this->dom->createElement('formField', $value);
			$field->setAttribute('name', $name);
			$form->appendChild($field);
		}
	}

	// Supprime tous les noeud fils de type formField d'un noeud form
	public function deleteFields($form)
	{
		$formFieldList = $this->findNodeList("formField", $this->xpath, $form);
		foreach($formFieldList as $formField)
			$form->removeChild($formField);
	}
}
?>