<?php
/**
* construit le texte XML de l'arbre des partenaires  partir de la liste des partenaires racine et de la liste des fils
*
* @param array(['PAR_ID'], ['PAR_NOM']) $tabRacine liste de tous les partenaires ayant un/des fil(s)
* @param array(['PAR_ID'], ['PAR_NOM'], ['GRP_ID']) $tabFils liste de tous les partenaires ayant un pre
* @param string $grpIcon chemin vers l'icone qui sera utilise pour reprsenter les partenaires groupes
* @param string $oneIcon chemin vers l'icone qui sera utilise pour reprsenter les partenaires feuilles
* @return string chane contenant le texte du XML
*/
function treeCreate($tabRacine, $tabFils, $grpIcon, $oneIcon)
{
	$tmp = "";
	
	$tabRacine = checkOrphelin($tabRacine, $tabFils);
	
	//Pour chaque racine on construit ses fils (partenaires dont le GRP_ID = ID de la racine)
	foreach($tabRacine as $racine)
	{
		$tmp .= '<row><item caption="'.$racine['PAR_NOM'].'" icon="'.($racine['PAR_TYPE'] == 1?$grpIcon: $oneIcon).'" value="'.$racine['PAR_ID'].'"/>';
		$nbFils = count($tabFils);
		$tmp .= add_noeud($racine['PAR_ID'], $tabFils, "", $grpIcon, $oneIcon, $nbFils);
		$tmp .= '</row>';
	}
	return $tmp;
}


/**
* dplace les fils de tabFils, dont le pre n'est pas dans tabRacine, dans tabRacine 
*
* @param array(['PAR_ID'], ['PAR_NOM']) $tabRacine liste de tous les partenaires ayant un/des fil(s)
* @param array(['PAR_ID'], ['PAR_NOM'], ['GRP_ID']) $tabFils liste de tous les partenaires ayant un pre
* @return array(['PAR_ID'], ['PAR_NOM']) liste $tabRacine mise  jour
*/
function checkOrphelin ($tabRacine, $tabFils)
{
	$isTabRacineModif = false;
	
	foreach($tabFils as $fils)
	{
		$notInRacine = true;
		$notInFils = true;
		
		foreach ($tabRacine as $racine)
		{
			if ($fils['GRP_ID'] == $racine['PAR_ID'])
			{
				$notInRacine = false;
				break;
			}
		}
		foreach ($tabFils as $filsRacine)
		{
			if ($fils['GRP_ID'] == $filsRacine['PAR_ID'])
			{
				$notInFils = false;
				break;
			}
		}
		
		if($notInRacine && $notInFils)
		{
			$tabRacine[] = $fils;
			$isTabRacineModif = true;
		}
	}
	
	//tri de tabRacine par ordre alphabtique des PAR_NOM s'il a t modifi
	if ($isTabRacineModif)
	{
		foreach ($tabRacine as $key => $row)
		{
			$parNom[$key]  = $row['PAR_NOM'];
			$parID[$key] = $row['PAR_ID'];
		}
		array_multisort($parNom, SORT_ASC, $parID, SORT_ASC, $tabRacine);
	}
	return $tabRacine;
}

/**
* construit le texte XML d'un noeud de l'arbre (un partenaire) et de ses fils
*
* @param string $grpId numro de groupe du partenaire (GRP_ID)
* @param array(['PAR_ID'], ['PAR_NOM'], ['GRP_ID']) $tabFils liste de tous les partenaires ayant un pre
* @param string $grpIcon chemin vers l'icone qui sera utilise pour reprsenter les partenaires groupes
* @param string $oneIcon chemin vers l'icone qui sera utilise pour reprsenter les partenaires feuilles
* @return string chane contenant le texte du XML
*/
function add_noeud ($grpId, &$tabFils, $ret, $grpIcon, $oneIcon, &$nbFils)
{
	$ret="";
	while(true)
	{
		$i=0;
		while (($i < $nbFils) && ($tabFils[$i]['GRP_ID'] < $grpId))
		{
			$i++;	
		}
		if ($i == $nbFils) 
		{
			$i--;
			return $ret;
		}
		if ($tabFils[$i]['GRP_ID'] > $grpId) return $ret;
		while (($i < $nbFils) && ($tabFils[$i]['GRP_ID'] == $grpId))
		{
			$fils=$tabFils[$i];
			array_splice($tabFils,$i,1);
			$nbFils--;
			$filsRet = add_noeud($fils['PAR_ID'], $tabFils, "", $grpIcon, $oneIcon, $nbFils);
			$ret .= '<row><item caption="'.$fils['PAR_NOM'].'" icon="'.($fils['PAR_TYPE'] == 1?$grpIcon: $oneIcon).'" value="'.$fils['PAR_ID'].'"/>';
			$ret .= $filsRet;
			$ret .= '</row>';
		}
	}
	return $ret;
}
?>