<?php
/***********************************
* Analyseur pour edition de macros *
* BP 061003 : Modif ajout des macro FICIN, FICOUT, FICRES *
***********************************/

//Analyse syntaxique
function Analyse_Grammaire($txt_macro, &$msg_err, $pos)
{
	//Chaine vide
	if (strlen(trim($txt_macro)) == 0) 
	{
		/*$msg_err = "Grammaire incorrecte (chaine vide ?) [M001]"; 
		return -1;*/
		return 0;
	}
	
	//On est en fin de chaine --> c'est fini
	if ($pos == strlen($txt_macro)) return $pos;
	
	//chaine de type TEXTE ?
	$res = Analyse_Texte($txt_macro, $msg_err, $pos);
	if ($res != $pos) 
	{
		//c'est du TEXTE --> STOP, on continue l'analyse
		$pos = $res; 
		return Analyse_Grammaire($txt_macro, $msg_err, $pos);
	}
	
	//chaine de type TEXTE SPECIAL ?
	$res = Analyse_Special_Texte($txt_macro, $msg_err, $pos);
	if ($res != $pos) 
	{
		//c'est du TEXTE SPECIAL --> STOP, on continue l'analyse
		$pos = $res; 
		return Analyse_Grammaire($txt_macro, $msg_err, $pos);
	}
	
	//chaine de type MACRO ?
	$res = Analyse_Macro($txt_macro, $msg_err, $pos);
	if ($res != -1) 
	{
		//c'est une MACRO --> STOP, on continue l'analyse
		$pos = $res; 
		return Analyse_Grammaire($txt_macro, $msg_err, $pos);
	}
	
	//chaine non reconnue ou incorrecte --> STOP, ERREUR
	return -1;
}

//Analyse des chaines de type TEXTE
function Analyse_Texte($txt_macro, &$msg_err, $pos)
{
	if ($pos < strlen($txt_macro) && (!preg_match("/[#]|[{]|[}]|[.]|[,]/", $txt_macro[$pos]))) 
		return Analyse_Texte($txt_macro, $msg_err, ++$pos);
	else 
		return ($pos);
}

//Analyse des chaines de type SPECIAL TEXTE
function Analyse_Special_Texte($txt_macro, &$msg_err, $pos)
{
	if($pos < strlen($txt_macro))
	{
		if (!preg_match("/[{]/", $txt_macro[$pos+1]))
			return Analyse_Texte($txt_macro, $msg_err, ++$pos);
		else
			return ($pos);
	}
	else 
		return ($pos);
}

//Analyse des chaines de type TEXTE avec point
function Analyse_Texte_Point($txt_macro, &$msg_err, $pos)
{
    if (!preg_match("/[#]|[{]|[}]|[,]/", $txt_macro[$pos]) && $pos < strlen($txt_macro)) 
        return Analyse_Texte_Point($txt_macro, $msg_err, ++$pos);
    else 
        return ($pos);
}


//Analyse des chaines de type entier
function Analyse_Entier($txt_macro, &$msg_err, $pos)
{
	if (preg_match("/[0-9]/", $txt_macro[$pos])) 
		return Analyse_Entier($txt_macro, $msg_err, ++$pos);
	else 
		return ($pos);
}

//Analyse des chaines de type DATE
function Analyse_Date($txt_macro, &$msg_err, $pos)
{
	if (preg_match("/[S]{2,2}|[A]{2,2}|[M]{2,2}|[J]{2,2}|[h]{2,2}|[m]{2,2}|[s]{2,2}/", substr($txt_macro, $pos, 2))) 
		return Analyse_Date($txt_macro, $msg_err, $pos+2);
	else 
		if (preg_match("/[Q]{3,3}/", substr($txt_macro, $pos, 3))) 
			return Analyse_Date($txt_macro, $msg_err, $pos+3);
		else
			if (preg_match("/[S]{1}|[A]{1}|[M]{1}|[J]{1}|[h]{1}|[m]{1}|[s]{1}/", substr($txt_macro, $pos, 1))) 
				return Analyse_Date($txt_macro, $msg_err, $pos+1);
			else 
				return ($pos);
}

//Analyse des chaines de type MACRO
// ${int.int} ou ${DATECOUR[,<format>]}
function Analyse_Macro($txt_macro, &$msg_err, $pos)
{
	global $lang_lbl_syntax_err,
	$lang_flux_gest_act_macro_msg_no_diese,
	$lang_flux_gest_act_macro_msg_no_open_brace,
	$lang_flux_gest_act_macro_msg_no_chain,
	$lang_flux_gest_act_macro_msg_no_date,
	$lang_flux_gest_act_macro_msg_no_int,
	$lang_flux_gest_act_macro_msg_no_int_n,
	$lang_flux_gest_act_macro_msg_no_int_m,
	$lang_flux_gest_act_macro_msg_no_point,
	$lang_flux_gest_act_macro_msg_no_close_brace;
	
	if ($txt_macro[$pos++] != "#") 
	{
		$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_diese";
		return -1;
	}
	if ($txt_macro[$pos++] != "{") 
	{
		$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_open_brace";
		return -1;
	}
	$temp = $pos;
	if (($res = Analyse_Texte($txt_macro, $msg_err, $pos)) == $pos) 
	{
		$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_chain";
		return -1;
	}
	$act = $temp.":".substr($txt_macro, $temp, $res-$temp);
	
	$buf = substr($txt_macro, $temp, $res-$temp);
	switch ($buf)
	{
		case "PART" :
			$pos = $pos + 5;
			if($txt_macro[$pos] == "#") // macro en paramètre
			{
				if (($res = Analyse_Macro($txt_macro, $msg_err, $pos)) == $pos) // peut contenir une macro...
				{
					$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_chain";
					return -1;
				}
			}
			else if (($res = Analyse_Texte($txt_macro, $msg_err, $pos)) == $pos)
			{
				$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_chain";
				return -1;
			}
			break;
		case "DATECOUR" :
		case "DATEPREC" :
			$pos = $res;
				if ($txt_macro[$pos++] == ",") 
					if (($res = Analyse_Date($txt_macro, $msg_err, $pos)) == $pos)
					{
						$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_date";
						return -1;
					}
				break;
		case "FLUX" :
		case "EXEC" :
		case "FICIN" :
		case "FICOUT" :
		case "FICRES" :
		case "EMET" :
		case "DEST" : 
				$pos = $res;
				break;
		//Macro de type variable
		default :
			if (preg_match("/CPT/", $buf))
			{
				$pos = $pos + 3;
				if (($res = Analyse_Entier($txt_macro, $msg_err, $pos)) == $pos)
				{
					$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int";
					return -1;
				}
				if (substr($txt_macro, $pos, $res-$pos) > 99999)
				{
					$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int_n";
						return -1;
				}
				$pos = $res;
				if ($txt_macro[$pos++] == ",") 
					if (($res = Analyse_Entier($txt_macro, $msg_err, $pos)) == $pos)
					{
						$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int";
						return -1;
					}
					if (substr($txt_macro, $pos, $res-$pos) > 255)
					{
						$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int_m";
						return -1;
					}
				break;
			
			}
            if (preg_match("/SCRIPT/", $buf))
            {
                $pos = $pos + 7;
                if (($res = Analyse_Texte($txt_macro, $msg_err, $pos)) == $pos)
                {
                    $msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_chain";
                    return -1;
                }
                if (substr($txt_macro, $pos, $res-$pos) > 99999)
                {
                    $msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_chain";
                        return -1;
                }
                $pos = $res;
                if ($txt_macro[$pos++] == ",") 
                {
	                if (($res = Analyse_Texte_Point($txt_macro, $msg_err, $pos)) == $pos)
	                {
	                    $msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_chain";
	                    return -1;
	                }
	                if (substr($txt_macro, $pos, $res-$pos) > 99999)
	                {
	                    $msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_chain";
	                        return -1;
	                }
                }
                else
                {
                    $msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_chain";
                    return -1;
                }
                break;
            }
			if (!preg_match("/^[0-9]{1,}$/", substr($txt_macro, $temp, $res-$temp)))
			{
				$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int";
				return -1;
			}
			$pos = $res;
			if ($txt_macro[$pos++] != ".") 
			{
				$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_point";
				return -1;
			}
			$temp = $pos;
			if (($res = Analyse_Texte($txt_macro, $msg_err, $pos)) == $pos) 
			{
				if (($res = Analyse_Entier($txt_macro, $msg_err, $pos)) == $pos) 
				{
					$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int";
					return -1;
				}
			}
			else
			{
				$txt = substr($txt_macro, $temp, $res-$temp);
				if (($txt != "FIC_ID") && ($txt != "S_LIBELLE") && ($txt != "S_NUMERR"))
				{
					$pos = $temp;
					if (($res = Analyse_Entier($txt_macro, $msg_err, $pos)) == $pos) 
					{
						$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int";
						return -1;
					}
				}
			}
			$par = $temp.":".substr($txt_macro, $temp, $res-$temp);
		
			$pos = $res;
			if ($txt_macro[$pos++] == ",") 
			{
				if (($res = Analyse_Entier($txt_macro, $msg_err, $pos)) == $pos)
				{
					$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int";
						return -1;
				} 
				if (substr($txt_macro, $pos, $res-$pos) > 255)
				{
					$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int_m";
						return -1;
				} 
				$pos = $res;
				if ($txt_macro[$pos++] == ",") 
					if (($res = Analyse_Entier($txt_macro, $msg_err, $pos)) == $pos)
					{
						$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int";
						return -1;
					}
					if (substr($txt_macro, $pos, $res-$pos) > 255)
					{
						$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_int_m";
							return -1;
					} 
			}
			$msg_err .= $act.":".$par."/";
	}
	
	$pos = $res;
	if ($txt_macro[$pos++] != "}") 
	{
		$msg_err = "$lang_lbl_syntax_err : $lang_flux_gest_act_macro_msg_no_close_brace";
		return -1;
	}
	
	return $pos;
}

//Analyse s�mantique
function verif_macro($txt_macro, $macros, $flux_id, $afx_num, &$msg_err)
{
	global $gDb, $lang_lbl_semantic_err,
	$lang_flux_gest_act_macro_msg_wrong_act,
	$lang_flux_gest_act_macro_msg_wrong_param;
	
	if ($macros == null || $macros == "") return 0;
	$macro = explode("/", $macros);

	foreach($macro as $mr)
	{
		$mc = explode(":", $mr);
		$act_pos 	= $mc[0];
		$act 		= $mc[1];
		$par_pos	= $mc[2];
		$par		= $mc[3];
		
		$req = "SELECT COUNT(AFX_ID) AS NB 
				FROM ACTFLUX 
				WHERE AFX_ID = $act 
				AND AFX_ACTIF != ".ETAT_SUPPR."
				AND FLX_ID = $flux_id 
				AND AFX_NUM < $afx_num";
		$gDb->SQLQuery($req, 5);
		$row = $gDb->SQLNextRow(5);
		if ($row['NB'] == 0) 
		{
			$gDb->SQLFermeQuery(5);
			$msg_err = "$lang_lbl_semantic_err : ".str_replace("%ACTNAME%", $act, $lang_flux_gest_act_macro_msg_wrong_act);
			return -1;
		}
		$gDb->SQLFermeQuery(5);

		if (($par != "FIC_ID") && ($par != "S_LIBELLE") && ($par != "S_NUMERR"))
		{
			$req = "SELECT COUNT(PFL_ID) AS NB 
					FROM PAR_FLOW, ACTIVITE, ACTFLUX 
					WHERE PFL_ID = $par 
					AND AFX_ID = $act
					AND ACTFLUX.ACT_ID = ACTIVITE.ACT_ID 
					AND ACTIVITE.TAC_NUM = PAR_FLOW.TAC_NUM";
			$gDb->SQLQuery($req, 5);
			$row = $gDb->SQLNextRow(5);
			if ($row['NB'] == 0) 
			{
				$gDb->SQLFermeQuery(5);
				$msg_err = "$lang_lbl_semantic_err : ".str_replace("%PARMNAME%", $par, $lang_flux_gest_act_macro_msg_wrong_param);
				return -1;
			}
		}
		$gDb->SQLFermeQuery(5);
	}
	
	return 0;
}
?>