<?php

// Definition des constantes pour les purposes des certificats
define ("cpUndefined",			   0);
define ("cpSSLClient",			   1);
define ("cpSSLCA",				   2);
define ("cpSSLServer",			   4);
define ("cpSSLServerCA",		   8);
define ("cpNetscapeSSLServer",	  16);
define ("cpNetscapeSSLServerCA",  32);
define ("cpSMIMESigning",		  64);
define ("cpSMIMESigningCA",	     128);
define ("cpSMIMEEncryption",	 256);
define ("cpSMIMEEncryptionCA",	 512);
define ("cpCRLSigning",			1024);
define ("cpCRLSigningCA",		2048);
define ("cpAnyPurpose",			4096);
define ("cpAnyPurposeCA",		8192);
define ("cpOCSPHelper",		   16384);
define ("cpOCSPHelperCA",	   32768);


// Extrait le nom court d'un DN
function ExtractName(&$name_chain)
{
	$short_name = "";
	// Rechercher le CN
	$tmp = explode("CN=",$name_chain);
	if(count($tmp)>1)
	{
		$short_name = explode ("/",$tmp[1]);
		$short_name = $short_name[0];
		if ($short_name == "")
		{
			// Rechercher l'OU
			$tmp = explode("OU=",$name_chain);
			$short_name = explode ("/",$tmp[1]);
			$short_name = $short_name[0];
			if ($short_name == "")
			{
				//Rechercher l'O
				$tmp = explode("O=",$name_chain);
				$short_name = explode ("/",$tmp[1]);
				$short_name = $short_name[0];
			}
		}
	}
	return $short_name;
}

// Extrait l'adresse mail d'un DN
function ExtractEMail(&$name_chain)
{
	$email_address = "";
	// Rechercher l'adresse
	$tmp = explode("emailAddress=",$name_chain);
	if(count($tmp)>1)
	{
		$email_address = explode ("/",$tmp[1]);
		$email_address = $email_address[0];
		if (!$email_address)
		{
			// Rechercher l'E
			$tmp = explode("E=",$name_chain);
			$email_address = explode ("/",$tmp[1]);
			$email_address = $email_address[0];
		}
	}
	return $email_address;
}

// Determine si un certificat est de type CA
function CertificateIsCA(&$purpose)
{
	$CAFlag = cpSSLCA
	                + cpSSLServerCA
					+ cpNetscapeSSLServerCA
					+ cpSMIMESigningCA
					+ cpSMIMEEncryptionCA
					+ cpCRLSigningCA
					//+cpAnyPurposeCA
					+ cpOCSPHelperCA;
	if (($purpose & $CAFlag) != 0)
	{
		return true;
	}
	return false;	
}

// Retourne sous forme textuelle l'etat d'un certificat
function CertificateStatus(&$crtId, &$crlIdList)
{
	global $icon, $pathStyle,
		   $lang_flux_pargen_crt_aff_lbl_certificat_valide,
		   $lang_flux_pargen_crt_aff_lbl_chaine_de_confiance,
		   $lang_flux_pargen_crt_aff_err_certificat_introuvable,
		   $lang_flux_pargen_crt_aff_err_certificat_perime,
		   $lang_flux_pargen_crt_aff_err_certificat_revoque,
		   $lang_flux_pargen_crt_aff_err_acces_banque;
	// Recuperer l'etat du certificat
	$status = php_aocl_check_certificate($crtId,$crlIdList);
	// Fournir un libelle explicite dependant du status
	switch ($status)
	{
		case 0:
			// Certificat valide
			$etat_certif = $lang_flux_pargen_crt_aff_lbl_certificat_valide; 
			break;
		case -2:
			// Le certificat en base UIF n'est pas trouve dans la banque
			$etat_certif = $lang_flux_pargen_crt_aff_err_certificat_introuvable; 
			$icon = $pathStyle."imgs/icons/error.png";
			break;
		case -3:
			// Le certificat est perime
			$etat_certif = $lang_flux_pargen_crt_aff_err_certificat_perime; 
			$icon = $pathStyle."imgs/icons/error.png";
			break;
		case -4:
			// Le certificat est revoque
			$etat_certif = $lang_flux_pargen_crt_aff_err_certificat_revoque; 
			$icon = $pathStyle."imgs/icons/error.png";
			break;
		case -5:
			// La chaine de confiance du certificat ne peut pas etre verifiee
			$etat_certif = $lang_flux_pargen_crt_aff_lbl_chaine_de_confiance; 
			$icon = $pathStyle."imgs/icons/alert.gif";
			break;
		case -1:
		default:
			// Impossible d'acceder a la banque de certificats
			$etat_certif = $lang_flux_pargen_crt_aff_err_acces_banque; 
			$icon = $pathStyle."imgs/icons/error.png";
			break;
	}
	return $etat_certif;
}

?>