<?php
define("PHPSVN_DIR", dirname(__FILE__));

require_once PHPSVN_DIR . "/http.php";
require_once PHPSVN_DIR . "/definitions.php";
require_once PHPSVN_DIR . "/xml2Array.php";

//  PHP SVN CLIENT
//  Uitlisation de la couche Web-DAV pour acc�der � un serveur SVN
class phpsvnclient
{
    private $_url;
    private $_http;
    private $_repVersion; // Numéro de révision du Respository
    private $pass;
    private $user;

    public $errNro; // Valeurs possibles : NOT_ERROR, NOT_FOUND, AUTH_REQUIRED, UNKOWN_ERROR

    private $actVersion; // révision actuelle
    private $storeDirectoryFiles = array();
    private $lastDirectoryFiles;
	private $file_size_founded;

    public function phpsvnclient($url = '', $user = false, $pass = false)
	{
        $http = & $this->_http;
        $http = new http_class;
        $http->user_agent = "phpsvnclient";

        $this->_url = $url;
        $this->user = $user;
        $this->pass = $pass;

        $this->actVersion = $this->getVersion();

        //register_shutdown_function(array(&$this, '__destruct'));
    }

	// Dump des données SVN pour $folder dans la révision $version (-1 = actuel) du repository
    public function rawDirectoryDump($folder = '/', $version = -1)
	{
        if ($version == -1 || $version > $this->actVersion)
            $version = $this->actVersion;

		$url = $this->cleanURL($this->_url . "/!svn/bc/" . $version . "/" . $folder . "/");
        $this->initQuery($args, "PROPFIND", $url);
        $args['Body'] = PHPSVN_NORMAL_REQUEST;
        $args['Headers']['Content-Length'] = strlen(PHPSVN_NORMAL_REQUEST);

        if (!$this->Request($args, $headers, $body))
            return false;

        $xml2Array = new xml2Array();
        return $xml2Array->xmlParse($body);
    }

	//  Retourne tous les fichiers de $folder dans la révision $version (-1 = actuel)
    public function getDirectoryFiles($folder = '/', $version = -1)
	{
        if ($arrOutput = $this->rawDirectoryDump($folder, $version))
		{
            $files = array();
            foreach ($arrOutput['children'] as $key => $value)
			{
                array_walk_recursive($value, array($this, 'storeDirectoryFiles'));
                array_push($files, $this->storeDirectoryFiles);
                unset($this->storeDirectoryFiles);
            }
            return $files;
        }
        return false;
    }

	// Retourne l'arbre complet de fichiers et dossiers contenus dans $folder à la révision $version (-1 = actuel)
	// Peut être utilisé aussi pour avoir les infos sur un seul fichier ou dossier si $recursive = false
    public function getDirectoryTree($folder = '/', $version = -1, $recursive = true)
	{
        $directoryTree = array();

        if (!($arrOutput = $this->getDirectoryFiles($folder, $version)))
            return false;

        if (!$recursive)
            return $arrOutput[0];

        while (count($arrOutput) && is_array($arrOutput))
		{
            $array = array_shift($arrOutput);

            array_push($directoryTree, $array);

            if (trim($array['path'], '/') == trim($folder, '/'))
                continue;

            if ($array['type'] == 'directory')
			{
                $walk = $this->getDirectoryFiles($array['path'], $version);
                array_shift($walk);

                foreach ($walk as $step)
				    array_unshift($arrOutput, $step);
            }
        }
        return $directoryTree;
    }

	// Retourne le contenu d'un fichier ou false en cas d'erreur ou si c'est un dossier
    public function getFile($file, $version = -1)
	{
        if ($version == -1 || $version > $this->actVersion)
            $version = $this->actVersion;

        // Si dossier return false
		// Sinon récupère le rendu via HTTP du serveur SVN
        $fileInfo = $this->getDirectoryTree($file, $version, false);
        if ($fileInfo["type"] == "directory")
            return false;

        $url = $this->cleanURL($this->_url . "/!svn/bc/" . $version . "/" . $file . "/");
        $this->initQuery($args, "GET", $url);
        if (!$this->Request($args, $headers, $body))
            return false;

        return $body;
    }

	// Renvoi les changements d'un fichier entre 2 révisions
    public function getFileLogs($file, $vini = 0, $vend = -1)
	{
        $fileLogs = array();

        if ($vend == -1 || $vend > $this->actVersion)
            $vend = $this->actVersion;

        if ($vini < 0)
            $vini = 0;
        if ($vini > $vend)
            $vini = $vend;

        $url = $this->cleanURL($this->_url . "/!svn/bc/" . $this->actVersion . "/" . $file . "/");
        $this->initQuery($args, "REPORT", $url);
        $args['Body'] = sprintf(PHPSVN_LOGS_REQUEST, $vini, $vend);
        $args['Headers']['Content-Length'] = strlen($args['Body']);
        $args['Headers']['Depth'] = 1;

        if (!$this->Request($args, $headers, $body))
            return false;

        $xml2Array = new xml2Array();
        $arrOutput = $xml2Array->xmlParse($body);

        foreach ($arrOutput['children'] as $value)
		{
            $array = array();
            foreach ($value['children'] as $entry)
			{
				if(!isset($entry['tagData'])) $entry['tagData'] = '';
				
                if ($entry['name'] == 'D:VERSION-NAME')
                    $array['version'] = $entry['tagData'];
                if ($entry['name'] == 'D:CREATOR-DISPLAYNAME')
                    $array['author'] = $entry['tagData'];
                if ($entry['name'] == 'S:DATE')
                    $array['date'] = $entry['tagData'];
                if ($entry['name'] == 'D:COMMENT')
                    $array['comment'] = $entry['tagData'];

                if (($entry['name'] == 'S:ADDED-PATH') || ($entry['name'] == 'S:MODIFIED-PATH') || ($entry['name'] == 'S:DELETED-PATH'))
				{
                    $array['files'][] = $entry['tagData'];

                    if ($entry['name'] == 'S:ADDED-PATH')
                        $array['add_files'][] = $entry['tagData'];
                    if ($entry['name'] == 'S:MODIFIED-PATH')
                        $array['mod_files'][] = $entry['tagData'];
                    if ($entry['name'] == 'S:DELETED-PATH')
                        $array['del_files'][] = $entry['tagData'];
                }
            }
            array_push($fileLogs, $array);
        }

        return $fileLogs;
    }
	
	// Extrait la révision du tag correspondant (D:HREF) si présent
	private function extractVersion($value)
	{
		$t = explode("/", $value);

		// on part de la fin si non vide
		$index = count($t) - 1;
		while ($t[$index] == "")
			$index--;

		// La révision est toujours à la fin, si elle y est...
		if (is_numeric($t[$index]))
		{
			$this->_repVersion = $t[$index];
			return true;
		}
		
		return false;
	}
	
	// Recherche récursive du numéro de révision sur l'arbre XML retourné par SVN
	private function analyseVersion($node)
	{
		if(is_array($node))
		{
			if(!strcasecmp($node['name'],"D:HREF") && $node['tagData'])
				return $this->extractVersion($node['tagData']);
			else if(is_array($node['children']) && count($node['children'])>0)
			{
				foreach($node['children'] as $child)
				{
					if($this->analyseVersion($child))
						return true;;
				}
			}
		}
		return false;
	}

	// Retourne le numéro de révision du Repository
    public function getVersion() {
        if ($this->_repVersion > 0)
            return $this->_repVersion;

        $this->_repVersion = -1;
        $this->initQuery($args, "PROPFIND", $this->cleanURL($this->_url . "/!svn/vcc/default"));
        $args['Body'] = PHPSVN_VERSION_REQUEST;
        $args['Headers']['Content-Length'] = strlen(PHPSVN_NORMAL_REQUEST);
        $args['Headers']['Depth'] = 0;

        if (!$this->Request($args, $tmp, $body)) {
            return $this->_repVersion;
        }
		
		$xml2Array = new xml2Array();
        $xmlTab = $xml2Array->xmlParse($body);
		if(is_array($xmlTab['children']) && count($xmlTab['children'])>0)
		{
			foreach($xmlTab['children'] as $child)
			{
				if($this->analyseVersion($child))
					break;
			}
		}
        return $this->_repVersion;
    }

	// Fonction Callback pour array_walk_recursive utilisé par getDirectoryFiles
    private function storeDirectoryFiles($item, $key)
	{
        if ($key == 'name')
		{
            if (($item == 'D:HREF') ||
                    ($item == 'LP1:GETLASTMODIFIED') ||
                    ($item == 'LP1:VERSION-NAME') ||
                    ($item == 'LP2:BASELINE-RELATIVE-PATH') ||
                    ($item == 'LP3:BASELINE-RELATIVE-PATH') ||
                    ($item == 'D:STATUS'))
			{
                $this->lastDirectoryFiles = $item;
            }
        }
		elseif (($key == 'tagData') && ($this->lastDirectoryFiles != ''))
		{
            // Pas sur que ce soit le bon D:HREF mais ça à l'air de fonctionner...
            if (($this->lastDirectoryFiles == 'D:HREF') && (isset($this->storeDirectoryFiles['type'])))
                return;

            // Transformation en tableau 
            switch ($this->lastDirectoryFiles)
			{
                case 'D:HREF':
                    $var = 'type';
                    break;
				case 'LP1:VERSION-NAME':
					$var = 'version';
                    break;
                case 'LP1:GETLASTMODIFIED':
                    $var = 'last-mod';
                    break;
                case 'LP2:BASELINE-RELATIVE-PATH':
                case 'LP3:BASELINE-RELATIVE-PATH':
                    $var = 'path';
                    break;
                case 'D:STATUS':
                    $var = 'status';
                    break;
            }
            $this->storeDirectoryFiles[$var] = $item;
            $this->lastDirectoryFiles = '';

            // Detection du 'type' comme 'directory' ou 'file'
            if ((isset($this->storeDirectoryFiles['type'])) &&
                    (isset($this->storeDirectoryFiles['last-mod'])) &&
                    (isset($this->storeDirectoryFiles['path'])) &&
                    (isset($this->storeDirectoryFiles['status'])))
			{
                $this->storeDirectoryFiles['path'] = str_replace(' ', '%20', $this->storeDirectoryFiles['path']); //Hack to make filenames with spaces work.
                $len = strlen($this->storeDirectoryFiles['path']);
                if (substr($this->storeDirectoryFiles['type'], strlen($this->storeDirectoryFiles['type']) - $len) == $this->storeDirectoryFiles['path'])
				{
                    $this->storeDirectoryFiles['type'] = 'file';
                }
				else
				{
                    $this->storeDirectoryFiles['type'] = 'directory';
                }
            }
        }
		else
		{
            $this->lastDirectoryFiles = '';
        }
    }

	// Init du client HTTP
	// $method peut être GET, POST, PROPFIND, REPORT, ...
    private function initQuery(&$arguments, $method, $url)
	{
        $http = & $this->_http;
        $http->GetRequestArguments($url, $arguments);
        if (isset($this->user) && isset($this->pass))
		{
            $arguments["Headers"]["Authorization"] = " Basic " . base64_encode($this->user . ":" . $this->pass);
        }
        $arguments["RequestMethod"] = $method;
        $arguments["Headers"]["Content-Type"] = "text/xml";
        $arguments["Headers"]["Depth"] = 1;
    }

	// Ouvre la connexion, envoi la requete et lit le header et le body
    private function Request($args, &$headers, &$body)
	{
		if(!isset($args['RequestURI'])) // rien à faire car pas d'URL !
			return false;
			
        $args['RequestURI'] = str_replace(' ', '%20', $args['RequestURI']); // On remplace les espaces
        $http = & $this->_http;
        $http->Open($args);
        $http->SendRequest($args);
        $http->ReadReplyHeaders($headers);
        if ($http->response_status[0] != 2)
		{
            switch ($http->response_status)
			{
                case 404:
                    $this->errNro = NOT_FOUND;
                    break;
                case 401:
                    $this->errNro = AUTH_REQUIRED;
                    break;
                default:
                    $this->errNro = UNKNOWN_ERROR;
                    break;
            }
            $http->close();
            return false;
        }
        $this->errNro = NO_ERROR;
        $body = '';
        $tbody = '';
        for (;;)
		{
            $error = $http->ReadReplyBody($tbody, 1000);
            if ($error != "" || strlen($tbody) == 0)
			{
                break;
            }
            $body.= ( $tbody);
        }
        $http->close();
        return true;
    }

	// Supprimme les // de trop
    private function cleanURL($url)
	{
        return preg_replace("/((^:)\/\/)/", "//", $url);
    }

	// Retourne la taille d'un fichier ou d'un dossier à la révision $version (-1 = actuel)
    public function getFileSize($file = '/', $version = -1)
	{
        if ($version == -1 || $version > $this->actVersion)
            $version = $this->actVersion;

		$url = $this->cleanURL($this->_url . "/!svn/bc/" . $version . "/" . $file . "/");
        $this->initQuery($args, "PROPFIND", $url);
        $args['Body'] = PHPSVN_GET_FILE_SIZE;
        $args['Headers']['Content-Length'] = strlen(PHPSVN_GET_FILE_SIZE);

        if (!$this->Request($args, $headers, $body))
            return false;

		$xml2Array = new xml2Array();
        $arrOutput = $xml2Array->xmlParse($body);

        if ($arrOutput)
		{
            $files = array();
            foreach ($arrOutput['children'] as $key => $value)
                array_walk_recursive($value, array($this, 'get_file_size_resursively'));

			return $this->file_size;
        }
    }
	
	// Calcul de la taille d'un dossier en fonction de ses élements fils (récursif via array_walk_recursive)
    private function get_file_size_resursively($item, $key)
	{
        if ($key == 'name')
		{
            if ($item == 'LP1:GETCONTENTLENGTH')
                $this->file_size_founded = true;
        }
		elseif (($key == 'tagData') && $this->file_size_founded)
		{
            $this->file_size = $item;
            $this->file_size_founded = false;
        }
    }
}
?>